require 'atomic'
require 'forwardable'
require 'monitor'
require 'set'
require 'thread'

require 'contender/version'
require 'contender/errors'

require 'contender/copy_on_write_structure'
require 'contender/copy_on_write_array'
require 'contender/copy_on_write_set'
require 'contender/countdown_latch'
require 'contender/counter'
require 'contender/executor'
require 'contender/executor_service'
require 'contender/direct_executor'
require 'contender/future'
require 'contender/future_task'
require 'contender/pool'
require 'contender/thread_factory'

require 'contender/queue'
require 'contender/linked_queue'

module Contender
  # http://stackoverflow.com/questions/535721/ruby-max-integer
  FIXNUM_MAX = (2**(0.size * 8 - 2) - 1)
  FIXNUM_MIN = -(2**(0.size * 8 - 2))

  # @param [Integer] size
  # @param [Boolean] allow_timeout
  # @return [PoolExecutor]
  def self.fixed_pool(size, allow_timeout = false)
    executor = Pool::PoolExecutor.new size, size, 0, LinkedQueue.new, simple_thread_factory

    if allow_timeout
      executor.work_timeout = 60
      executor.allow_core_timeout = true
    end

    executor
  end

  # @return [PoolExecutor]
  def self.single_pool
    fixed_pool 1
  end

  def self.simple_thread_factory
    SimpleThreadFactory.new
  end
end

