module Contender
  # Array that uses copy-on-write to guarantee thread safety
  #
  # Use this when read throughput is the primary requirement; write operations will always be
  # slow due to the need to create a shallow copy of the array to perform operations on.
  class CopyOnWriteArray
    include CopyOnWriteStructure
    include Comparable
    include Enumerable

    delegate_to Array

    mutators :<<, :[]=, :clear, :collect!, :compact!, :concat, :delete, :delete_at, :delete_if,
      :fill, :flatten!, :insert, :keep_if, :map!, :pop, :push, :reject!, :replace, :reverse!,
      :rotate!, :select!, :shift, :shuffle!, :slice!, :sort!, :sort_by!, :uniq!, :unshift

    accessors!
  end # CopyOnWriteArray
end
