module Contender
  # @abstract
  module CopyOnWriteStructure
    extend Forwardable

    def self.included(receiver)
      receiver.send :extend, ClassMethods
      receiver.send :extend, Forwardable
    end

    module ClassMethods
      def accessors(*args)
        @accessors ||= Set.new

        args.each do |method|
          @accessors.add method

          def_delegator :current_reference, method
        end
      end

      def accessors!
        methods = Set.new(delegated_type.instance_methods(false))

        methods = methods - @accessors if @accessors
        methods = methods - @mutators if @mutators

        methods.each do |method|
          def_delegator :current_reference, method
        end
      end

      def mutators(*args)
        @mutators ||= Set.new

        args.each do |method|
          @mutators.add method

          class_eval <<-RUBY_EVAL, __FILE__, __LINE__ + 1
            def #{method}(*args, &block)
              update_reference do |current|
                current.#{method}(*args, &block)
              end
            end
          RUBY_EVAL
        end
      end

      alias_method :accessor, :accessors
      alias_method :mutator, :mutators

      def delegate_to(type)
        @delegated_type = type
      end

      attr_reader :delegated_type
    end

    def initialize(*args, &block)
      @reference = Atomic.new(self.class.delegated_type.new(*args, &block))
    end

    protected

    # @return [Object]
    def current_reference
      @reference.value
    end

    # @yield [Object]
    # @return [Object]
    def update_reference
      loop do
        current = current_reference
        update = current.clone

        result = yield update

        if result.equal? update
          # Most write operations return a reference to self
          # Don't let this reference leak to the caller
          result = self
        end

        return result if @reference.compare_and_swap current, update

        # Concurrent modification, retry operation
      end
    end
  end # CopyOnWriteStructure
end
