module Contender
  # Represents a mechanism for executing submitted blocks
  #
  # This interface decouples task submission from the way that tasks will be run, including details
  # of thread use, scheduling, etc.
  #
  # @abstract
  class Executor
    # Executes the given block at some time in the future
    #
    # The block may execute in a new thread, in a pooled thread, or in the calling thread, at the
    # discretion of the implementation.
    #
    # @abstract
    # @param [Object] task
    # @return [undefined]
    def execute(task = nil, &block)
      raise NotImplementedError
    end
  end # Executor
end
