module Contender
  # @abstract
  class ExecutorService < Executor
    # @abstract
    # @return [undefined]
    def shutdown
      raise NotImplementedError
    end

    # @abstract
    # @return [Array]
    def shutdown!
      raise NotImplementedError
    end

    # @abstract
    # @param [Float] timeout
    # @return [Boolean]
    def await_termination(timeout)
      raise NotImplementedError
    end

    # @abstract
    # @return [Boolean]
    def shutdown?
      raise NotImplementedError
    end

    # @abstract
    # @return [Boolean]
    def terminated?
      raise NotImplementedError
    end

    # @api public
    # @raise [ArgumentError] If neither a block nor callable were given
    # @param [Object] callable
    # @return [FutureTask]
    def submit(callable = nil, &block)
      callable ||= block

      raise ArgumentError unless callable

      future = future_for callable
      execute future
      future
    end

    # @api public
    # @param [Array<Object>] tasks
    # @return [Array<FutureTask>]
    def invoke_all(tasks)
      finished = false
      futures = Array.new

      begin
        tasks.each do |task|
          future = create_task_for task

          futures.push future
          execute future
        end

        futures.each do |future|
          unless future.done?
            begin
              future.result
            rescue CancellationError
            rescue ExecutionError
              # The caller can deal with these errors
            end
          end
        end

        finished = true
        futures
      ensure
        unless finished
          futures.each do |future|
            future.cancel true
          end
        end
      end
    end

    # @api public
    # @param [Object] task
    # @return [FutureTask]
    def future_for(task)
      FutureTask.new task
    end
  end
end

