module Contender
  # @abstract
  class Future
    # @abstract
    # @param [Boolean] should_interrupt
    # @return [Boolean] True if this future was cancelled
    def cancel(should_interrupt)
      raise NotImplementedError
    end

    # Returns true if this future was cancelled before it could complete
    #
    # @abstract
    # @return [Boolean]
    def cancelled?
      raise NotImplementedError
    end

    # Returns true if this future was either cancelled or completed
    #
    # @abstract
    # @return [Boolean]
    def done?
      raise NotImplementedError
    end

    # @abstract
    # @raise [ExecutionError] If the result of the operation was an exception
    # @raise [TimeoutError] If the timeout was reached before the operation completed
    # @raise [CancellationError] If the operation was cancelled
    # @param [Integer] timeout Time to wait for the result
    # @return [Object] The result of the future
    def result(timeout = nil)
      raise NotImplementedError
    end
  end
end
