module Contender
  module Pool
    class PoolWorker
      extend Forwardable

      attr_reader :completed_task_count
      attr_reader :thread

      def initialize(thread_factory, first_task)
        @mutex = Mutex.new

        @thread_factory = thread_factory
        @first_task = first_task
        @completed_task_count = 0
      end

      def first_task!
        @first_task.tap {
          @first_task = nil
        }
      end

      def on_task_completion
        @completed_task_count += 1
      end

      def start(&worker_loop)
        @thread = @thread_factory.create &worker_loop
      end

      def interrupt
        @thread.raise Interrupt
      end

      def_delegators :@mutex, :lock, :unlock, :try_lock, :locked?
    end # PoolWorker
  end # Pool
end
