module Contender
  # @abstract
  class Queue
    include Enumerable

    # @abstract
    # @param [Object] element
    # @param [Float] timeout
    # @return [Boolean]
    def offer(element, timeout = nil)
      raise NotImplementedError
    end

    # @abstract
    # @param [Object] element
    # @return [undefined]
    def put(element)
      raise NotImplementedError
    end

    # @abstract
    # @param [Float] timeout
    # @return [Object]
    def poll(timeout = nil)
      raise NotImplementedError
    end

    # @abstract
    # @return [Object]
    def take
      raise NotImplementedError
    end

    # @abstract
    # @return [Object]
    def peek
      raise NotImplementedError
    end

    # @abstract
    # @return [undefined]
    def clear
      raise NotImplementedError
    end

    # @abstract
    # @yield [Object]
    # @return [undefined]
    def each
      raise NotImplementedError
    end

    # @abstract
    # @param [Object] element
    # @return [Boolean]
    def delete(element)
      raise NotImplementedError
    end

    # @abstract
    # @param [Array] target
    # @param [Integer] max_elements
    # @return [Integer]
    def drain_to(target, max_elements = FIXNUM_MAX)
      raise NotImplementedError
    end

    # @abstract
    # @return [Integer]
    def capacity_remaining
      raise NotImplementedError
    end

    # @abstract
    # @return [Integer]
    def size
      raise NotImplementedError
    end

    alias_method :length, :size

    # @return [Boolean]
    def empty?
      size == 0
    end
  end
end
